'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _tls = require('tls');

var _tls2 = _interopRequireDefault(_tls);

var _stream = require('stream');

var _reconnectCore = require('reconnect-core');

var _reconnectCore2 = _interopRequireDefault(_reconnectCore);

var _ringbuffer = require('./ringbuffer');

var _ringbuffer2 = _interopRequireDefault(_ringbuffer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var newline = /\n/g;
var tokenPattern = /[a-f\d]{8}-([a-f\d]{4}-){3}[a-f\d]{12}/;

var connectedEvent = 'connected';
var disconnectedEvent = 'disconnected';
var timeoutEvent = 'timed out';
var bufferDrainEvent = 'buffer drain';

var finalizeLogString = function finalizeLogString(log, token) {
  return token + ' ' + log.toString().replace(newline, '\u2028') + '\n';
};

var Logger = function (_Writable) {
  (0, _inherits3.default)(Logger, _Writable);

  function Logger(opts) {
    (0, _classCallCheck3.default)(this, Logger);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Logger.__proto__ || (0, _getPrototypeOf2.default)(Logger)).call(this, {
      objectMode: true
    }));

    _this.secure = opts.secure || true;
    _this.debugLogEnabled = opts.debugLogEnabled || false;

    _this.bufferSize = 16192;
    _this.port = opts.port || (_this.secure ? 443 : 80);
    _this.host = opts.host || 'data.logentries.com';
    _this.inactivityTimeout = 15 * 1000;
    _this.token = opts.token;
    _this.reconnectInitialDelay = 1000;
    _this.reconnectMaxDelay = 15 * 1000;
    _this.reconnectBackoffStrategy = 'fibonacci';

    var isSecure = _this.secure;
    _this.ringBuffer = new _ringbuffer2.default(_this.bufferSize);
    _this.reconnect = (0, _reconnectCore2.default)(function () {
      var connection = void 0;
      var args = [{
        host: _this.host,
        port: _this.port
      }];
      if (isSecure) {
        connection = _tls2.default.connect.apply(_tls2.default, args, function () {
          if (!connection.authorized) {
            var errMsg = connection.authorizationError;
            _this.debugLog(errMsg);
          } else if (_tls2.default && _tls2.default.CleartextStream && connection instanceof _tls2.default.CleartextStream) {
            _this.emit('connect');
          }
        });
      } else {
        connection = net.connect.apply(null, args);
      }
      if (!opts.disableTimeout) {
        connection.setTimeout(_this.inactivityTimeout);
      }
      return connection;
    });

    _this.ringBuffer.on('buffer shift', function () {
      _this.debugLog('Buffer is full, will be shifting records until buffer is drained.');
    });

    _this.on(bufferDrainEvent, function () {
      _this.debugLog('RingBuffer drained.');
      _this.drained = true;
    });

    _this.on(timeoutEvent, function () {
      if (_this.drained) {
        _this.debugLog('Socket was inactive for ' + _this.inactivityTimeout / 1000 + ' seconds. Destroying.');
        _this.closeConnection();
      } else {
        _this.debugLog('Inactivity timeout event emitted but buffer was not drained.');
        _this.once(bufferDrainEvent, function () {
          _this.debugLog('Buffer drain event emitted for inactivity timeout.');
          _this.closeConnection();
        });
      }
    });
    return _this;
  }

  (0, _createClass3.default)(Logger, [{
    key: '_write',
    value: function _write(ch, enc, cb) {
      var _this2 = this;

      this.drained = false;
      this.connection.then(function (conn) {
        var record = _this2.ringBuffer.read();
        if (record) {
          if (_this2.ringBuffer.isEmpty()) {
            conn.write(record, function () {
              process.nextTick(function () {
                _this2.emit(bufferDrainEvent);
              });
            });
          } else {
            conn.write(record);
          }
        }
        cb();
      }).catch(function (err) {
        _this2.debugLog('Error: ' + err);
        cb();
      });
    }
  }, {
    key: 'log',
    value: function log(_log) {
      if (this.ringBuffer.write(finalizeLogString(_log, this.token))) {
        this.write();
      }
    }
  }, {
    key: 'closeConnection',
    value: function closeConnection() {
      this.debugLog('Closing retry mechanism along with its connection.');
      if (!this.reconnection) {
        this.debugLog('No reconnection instance found. Returning.');
        return;
      }
      this.reconnection.disconnect();
      this.connection = null;
    }
  }, {
    key: 'debugLog',
    value: function debugLog(message) {
      if (this.debugLogEnabled) {
        console.log(message);
      }
    }
  }, {
    key: 'connection',
    get: function get() {
      var _this3 = this;

      if (this._connection) {
        return this._connection;
      }

      this.debugLog('No connection exists. Creating a new one.');
      if (this.reconnection) {
        this.reconnection.disconnect();
        this.reconnection = null;
      }

      this.reconnection = this.reconnect({
        initialDelay: this.reconnectInitialDelay,
        maxDelay: this.reconnectMaxDelay,
        strategy: this.reconnectBackoffStrategy,
        failAfter: Infinity,
        randomisationFactor: 0,
        immediate: false
      });

      this.connection = new _promise2.default(function (resolve) {

        _this3.reconnection.on('connect', function (connection) {
          _this3.emit(connectedEvent);

          connection.on('timeout', function () {
            _this3.emit(timeoutEvent);
          });
          resolve(connection);
        });

        _this3.reconnection.on('reconnect', function (n, delay) {
          if (n > 0) {
            _this3.debugLog('Trying to reconnect. Times: ' + n + ' , previous delay: ' + delay);
          }
        });

        _this3.reconnection.once('disconnect', function () {
          _this3.debugLog('Socket was disconnected');
          _this3.connection = null;
          _this3.emit(disconnectedEvent);
        });

        _this3.reconnection.on('error', function (err) {
          _this3.debugLog('Error occurred during connection: ' + err);
        });

        _this3.reconnection.connect();
      });
      return this.connection;
    },
    set: function set(obj) {
      this._connection = obj;
    }
  }, {
    key: 'reconnection',
    get: function get() {
      return this._reconnection;
    },
    set: function set(func) {
      this._reconnection = func;
    }
  }]);
  return Logger;
}(_stream.Writable);

exports.default = Logger;
module.exports = exports['default'];
//# sourceMappingURL=logger.js.map
